package org.apidesign.apifest08.currency;


import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.util.Currency;


/**
 * Convert between two currencies.
 *
 * @author D'Arcy Smith
 * @version 1.0
 */
public final class Convertor
{
    /**
     * The currency to cvonvert from.
     */
    private final Currency currencyA;

    /**
     * The currency to cvonvert from.
     */
    private final Currency currencyB;
    
    /**
     * Constructs a convertor with the specified currencies.
     * 
     * @param a the currency to convert from.
     * @param b the currency to convert to.
     * @throws IllegalArgumentException if either a or b are null.
     */
    public Convertor(final Currency a,
                     final Currency b)
    {
        if(a == null)
        {
            throw new IllegalArgumentException("a cannot be null");
        }

        if(b == null)
        {
            throw new IllegalArgumentException("a cannot be null");
        }

        currencyA = a;
        currencyB = b;
    }
    
    /**
     * Convert from currency "b" to currency "a".
     * 
     * @param amount the amount to convert.
     * @return the converted amount.
     * @throws IllegalArgumentException if amount is null.
     */
    public BigDecimal convertFrom(final BigDecimal amount)
    {
        final BigDecimal aInUSD;
        final BigDecimal bInUSD;
        final BigDecimal temp;
        final BigDecimal result;
        
        if(amount == null)
        {
            throw new IllegalArgumentException("amount cannot be null");
        }
        
        aInUSD = CurrencyValues.getValue(currencyA);
        bInUSD = CurrencyValues.getValue(currencyB);
        temp   = amount.divide(bInUSD, MathContext.DECIMAL32);
        result = temp.multiply(aInUSD);
        
        return (result.setScale(2, RoundingMode.HALF_DOWN));
    }
    
    /**
     * Convert from currency "a" to currency "b".
     * 
     * @param amount the amount to convert.
     * @return the converted amount.
     * @throws IllegalArgumentException if amount is null.
     */
    public BigDecimal convertTo(final BigDecimal amount)
    {
        final BigDecimal aInUSD;
        final BigDecimal bInUSD;
        final BigDecimal temp;
        final BigDecimal result;
        
        if(amount == null)
        {
            throw new IllegalArgumentException("amount cannot be null");
        }
        
        aInUSD = CurrencyValues.getValue(currencyA);
        bInUSD = CurrencyValues.getValue(currencyB);
        temp      = amount.divide(aInUSD, MathContext.DECIMAL32);
        result = temp.multiply(bInUSD);
        
        return (result.setScale(2, RoundingMode.HALF_DOWN));
    }
}
