package org.apidesign.apifest08.currency;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Currency;
import static org.apidesign.apifest08.currency.Assert.*;

/**
 * An amount representation. Amount is represented as composition of a value and 
 * a currency.
 */
public final class Amount {
	
	private final BigDecimal value;
	private final Currency currency;
	private final int scale;
	private final RoundingMode roundingMode;
	
	public static final RoundingMode DEFAULT_ROUNDING = RoundingMode.HALF_EVEN; 
	
	public Amount(final BigDecimal value, final Currency currency) {
		notNull(value, "value");
		notNull(currency, "currency");
		this.value = value;
		this.currency = currency;
		this.scale = currency.getDefaultFractionDigits();
		this.roundingMode = DEFAULT_ROUNDING;
	}
	
	public Amount(final BigDecimal value, final Currency currency, final RoundingMode roundingMode) {
		notNull(value, "value");
		notNull(currency, "currency");
		notNull(roundingMode, "roundingMode");
		
		this.value = value;
		this.currency = currency;
		this.scale = currency.getDefaultFractionDigits();
		this.roundingMode = roundingMode;
	}
	
	public Amount(final long value, final Currency currency) {
		this(BigDecimal.valueOf(value), currency);
	}
	
	public Amount(final String value, final Currency currency) {
		this(new BigDecimal(value), currency);
	}
	
	/**
	 * @return the value with scale of the associated currency and rounded by 
	 * the rounding mode. 
	 */
	public BigDecimal getValue() {
		return value.setScale(scale, roundingMode);
	}
	
	/**
	 * @return the raw (no explicit scale, no explicit rounding) value 
	 */
	public BigDecimal getRawValue() {
		return value;
	}
	
	public Currency getCurrency() {
		return currency;
	}

	public int getScale() {
		return scale;
	}

	public RoundingMode getRoundingMode() {
		return roundingMode;
	}

	@Override
	public String toString() {
		return value + ",- " + currency.toString();
	}	
}
